Index: sys/netinet/tcp_sack.c
===================================================================
--- sys/netinet/tcp_sack.c	(revision 296480)
+++ sys/netinet/tcp_sack.c	(working copy)
@@ -355,12 +355,13 @@ tcp_sack_doack(struct tcpcb *tp, struct tcpopt *to
 {
 	struct sackhole *cur, *temp;
 	struct sackblk sack, sack_blocks[TCP_MAX_SACK + 1], *sblkp;
-	int i, j, num_sack_blks, sack_changed;
+	int i, j, num_sack_blks, sack_changed, sacked_bytes;
 
 	INP_WLOCK_ASSERT(tp->t_inpcb);
 
 	num_sack_blks = 0;
 	sack_changed = 0;
+	sacked_bytes = 0;
 	/*
 	 * If SND.UNA will be advanced by SEG.ACK, and if SACK holes exist,
 	 * treat [SND.UNA, SEG.ACK) as if it is a SACK block.
@@ -374,7 +375,6 @@ tcp_sack_doack(struct tcpcb *tp, struct tcpopt *to
 	 * received new blocks from the other side.
 	 */
 	if (to->to_flags & TOF_SACK) {
-		tp->sackhint.sacked_bytes = 0;	/* reset */
 		for (i = 0; i < to->to_nsacks; i++) {
 			bcopy((to->to_sacks + i * TCPOLEN_SACK),
 			    &sack, sizeof(sack));
@@ -387,8 +387,6 @@ tcp_sack_doack(struct tcpcb *tp, struct tcpopt *to
 			    SEQ_GT(sack.end, tp->snd_una) &&
 			    SEQ_LEQ(sack.end, tp->snd_max)) {
 				sack_blocks[num_sack_blks++] = sack;
-				tp->sackhint.sacked_bytes +=
-				    (sack.end-sack.start);
 			}
 		}
 	}
@@ -446,6 +444,7 @@ tcp_sack_doack(struct tcpcb *tp, struct tcpopt *to
 		temp = tcp_sackhole_insert(tp, tp->snd_fack,sblkp->start,NULL);
 		if (temp != NULL) {
 			tp->snd_fack = sblkp->end;
+			sacked_bytes += (sblkp->end - sblkp->start);
 			/* Go to the previous sack block. */
 			sblkp--;
 			sack_changed = 1;
@@ -462,11 +461,14 @@ tcp_sack_doack(struct tcpcb *tp, struct tcpopt *to
 			       SEQ_LT(tp->snd_fack, sblkp->start))
 				sblkp--;
 			if (sblkp >= sack_blocks && 
-			    SEQ_LT(tp->snd_fack, sblkp->end))
+			    SEQ_LT(tp->snd_fack, sblkp->end)) {
+				sacked_bytes += (sblkp->end - tp->snd_fack);
 				tp->snd_fack = sblkp->end;
+			}
 		}
 	} else if (SEQ_LT(tp->snd_fack, sblkp->end)) {
 		/* fack is advanced. */
+		sacked_bytes += (sblkp->end - tp->snd_fack);
 		tp->snd_fack = sblkp->end;
 		sack_changed = 1;
 	}
@@ -503,6 +505,10 @@ tcp_sack_doack(struct tcpcb *tp, struct tcpopt *to
 			/* Data acks at least the beginning of hole. */
 			if (SEQ_GEQ(sblkp->end, cur->end)) {
 				/* Acks entire hole, so delete hole. */
+				if (SEQ_GT(cur->start, th_ack))
+					sacked_bytes += (cur->end - cur->start);
+				else
+					sacked_bytes -= (sblkp->end - cur->end);
 				temp = cur;
 				cur = TAILQ_PREV(cur, sackhole_head, scblink);
 				tcp_sackhole_remove(tp, temp);
@@ -514,6 +520,8 @@ tcp_sack_doack(struct tcpcb *tp, struct tcpopt *to
 				continue;
 			} else {
 				/* Move start of hole forward. */
+				if (SEQ_GT(sblkp->start, th_ack))
+					sacked_bytes += (sblkp->end - cur->start); 
 				cur->start = sblkp->end;
 				cur->rxmit = SEQ_MAX(cur->rxmit, cur->start);
 			}
@@ -521,6 +529,7 @@ tcp_sack_doack(struct tcpcb *tp, struct tcpopt *to
 			/* Data acks at least the end of hole. */
 			if (SEQ_GEQ(sblkp->end, cur->end)) {
 				/* Move end of hole backward. */
+				sacked_bytes += (cur->end - sblkp->start);
 				cur->end = sblkp->start;
 				cur->rxmit = SEQ_MIN(cur->rxmit, cur->end);
 			} else {
@@ -528,6 +537,7 @@ tcp_sack_doack(struct tcpcb *tp, struct tcpopt *to
 				 * ACKs some data in middle of a hole; need
 				 * to split current hole
 				 */
+				sacked_bytes += (sblkp->end - sblkp->start);
 				temp = tcp_sackhole_insert(tp, sblkp->end,
 				    cur->end, cur);
 				if (temp != NULL) {
@@ -554,6 +564,7 @@ tcp_sack_doack(struct tcpcb *tp, struct tcpopt *to
 		else
 			sblkp--;
 	}
+	tp->sackhint.sacked_bytes += sacked_bytes;
 	return (sack_changed);
 }
 
